import logging
import os.path
from itertools import chain
from operator import attrgetter
from os.path import basename

from django.contrib.auth.decorators import login_required
from django.contrib.sites.models import get_current_site
from django.core.servers.basehttp import FileWrapper
from django.http import HttpResponse, HttpResponseForbidden
from django.shortcuts import render
from models import IosApp, AndroidApp

import settings as app_dist_settings


log = logging.getLogger(__name__)


@login_required
def index(request):
	ios_apps = IosApp.objects.filter(user_id__exact=request.user.id)
	android_apps = AndroidApp.objects.filter(user_id__exact=request.user.id)
	apps = list(chain(ios_apps, android_apps))
	apps.sort(key=attrgetter('build_date'), reverse=True)
	apps.sort(key=attrgetter('version'), reverse=True)
	apps.sort(key=attrgetter('operating_system'))
	apps.sort(key=attrgetter('name'))	
	
	return render(request, 'django_mobile_app_distribution/app_list.html', {
		'apps' : apps,
		'ios_identifier' : app_dist_settings.IOS,
		'site_url' : get_current_site(request).domain
		})


@login_required
def send_apk(request, app_id):
	"""                                                                         
    Send a file through Django without loading the whole file into              
    memory at once. The FileWrapper will turn the file object into an           
    iterator for chunks of 8KB.                                                 
    """
	android_app = None
	try:
		android_app = AndroidApp.objects.get(pk=app_id)
	except AndroidApp.DoesNotExist:
		return HttpResponse('App does not exist', status=404)
	
	if android_app.user.id != request.user.id:
		return HttpResponseForbidden('This is not your app')

	filename = os.path.join(app_dist_settings.MOBILE_APP_DISTRIBUTION_ANDROID_FILE_STORAGE_PATH, android_app.app_binary.name)

	wrapper = FileWrapper(file(filename))
	response = HttpResponse(wrapper)
	response['Content-Length'] = os.path.getsize(filename)
	response['Content-Type'] = app_dist_settings.MOBILE_APP_DISTRIBUTION_CONTENT_TYPES[android_app.operating_system]
	response['Content-Disposition'] = 'inline; filename=%s' % basename(filename)
	return response




from django.conf.urls import patterns, url
from django.contrib.auth import views as auth_views


urlpatterns = patterns('',
                       url(r'^login/$',
                           auth_views.login,
                           {'template_name': 'django_mobile_app_distribution/login.html'},
                           name='auth_login'),
                       url(r'^logout/$',
                           auth_views.logout,
                           {'template_name': 'django_mobile_app_distribution/logout.html'},
                           name='auth_logout'),
)


# -*- coding: utf-8 -*-
import uuid

from django.conf import settings
from django.contrib.sites.models import Site
from django.forms import ModelForm
from django.utils.translation import ugettext_lazy as _, string_concat

import settings as app_dist_settings
from models import IosApp


class IosAppAdminForm(ModelForm):

	class Meta:
		model = IosApp

	def __init__(self, *args, **kwargs):
		super(IosAppAdminForm, self).__init__(*args, **kwargs)
		filename = None
		try:
			app = kwargs['instance']
			filename = app.file_name
		except KeyError:
			hex = uuid.uuid4().hex[:16]
			filename = hex + '.ipa'
			self.fields['file_name'].initial = filename
		
		self.fields['file_name'].help_text = string_concat(
			'<span style=color:red;>',
			_("Don't change this file name!"),
			'</span><br />',
			_('Paste the file name and Ad Hoc URL into the Xcode Enterprise Ad Hoc dialog.'),
			'<br />',
			_('File name:'),
			' ',
			'<span style=color:red;>',
			filename,
			'</span><br />',
			_('Ad Hoc URL:'),
			' ',
			'<span style=color:red;>',
			'/'.join(s.strip('/') for s in (Site.objects.get_current().domain, settings.MEDIA_URL, app_dist_settings.MOBILE_APP_DISTRIBUTION_IOS_UPLOAD_TO_DIRECTORY_NAME, filename)),
			'</span>')


# -*- coding: utf-8 -*-
import logging

from django.contrib import admin, messages
from django.contrib.auth.models import User
from django.contrib.sites.models import get_current_site
from django.core.mail import EmailMessage
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from django_mobile_app_distribution.models import IosApp, AndroidApp

from forms import IosAppAdminForm
from models import UserInfo


log = logging.getLogger(__name__)


# Define an inline admin descriptor for UserInfo model
# which acts a bit like a singleton
class UserInfoInline(admin.StackedInline):
    model = UserInfo
    can_delete = False

class UserAdmin(admin.ModelAdmin):
    inlines = (UserInfoInline, )


class NotifiableModelAdmin(admin.ModelAdmin):
	actions = ['notify_client']

	def notify_client(self, request, queryset):
		for app in queryset.all():
			if app.user.email:
				email = EmailMessage()
				email.subject = _('Version %(app_version)s of %(app_name)s for %(os)s is available for download') % {
				'app_name' : app.name, 
				'app_version' : app.version, 
				'os': app.operating_system
				}
				email.body = _('Version %(app_version)s of %(app_name)s for %(os)s is available for download.\nPlease visit %(download_url)s to install your app.') % {
				'app_name' : app.name, 
				'app_version' : app.version, 
				'os': app.operating_system,
				'download_url' : '/'.join(s.strip('/') for s in (get_current_site(request).domain, reverse('django_mobile_app_distribution_index')))
				}
				email.to = [app.user.email]
				email.send(fail_silently=False)
				messages.add_message(request, messages.INFO, _('The user %(user_name)s was notified of %(app_name)s %(app_version)s availability.') % {'user_name' :app.user.username, 'app_name' : app.name, 'app_version': app.version} , fail_silently=True)
			else:
				messages.add_message(request, messages.ERROR, _('The user %s does not have an email address. Please add an email address and try again.') % app.user.username, fail_silently=True)
	notify_client.short_description = _('Notify clients of app availability')

class IosAppAdmin(NotifiableModelAdmin):

	form = IosAppAdminForm
	list_display = ('name', 'user', 'version', 'comment', 'build_date' )

	fieldsets = (
		(_('App info'), {
			'fields': ('user', 'name', 'version', 'comment')
		}),
		(_('Binary info'), {
			'fields': ('file_name', 'build_date', 'app_binary', 'app_plist')
		}),
	)

class AndroidAppAdmin(NotifiableModelAdmin):
	list_display = ('name', 'user', 'version', 'comment', 'build_date' )

	fieldsets = (
		(_('App info'), {
			'fields': ('user', 'name', 'version', 'comment')
		}),
		(_('Binary info'), {
			'fields': ('build_date', 'app_binary')
		}),
	)

admin.site.register(IosApp, IosAppAdmin)
admin.site.register(AndroidApp, AndroidAppAdmin)
admin.site.unregister(User)
admin.site.register(User, UserAdmin)


from django.conf.urls import patterns, url

urlpatterns = patterns('',
    url(r'^$', 'django_mobile_app_distribution.views.index', name='django_mobile_app_distribution_index'),
    url(r'^apk/(?P<app_id>\d{1,10})$', 'django_mobile_app_distribution.views.send_apk', name='django_mobile_app_distribution_send_apk'),
)


import logging

import django_mobile_app_distribution.settings as app_dist_settings
from django.contrib.auth.models import User
from django.contrib.sites.models import Site
from django.core.files.storage import FileSystemStorage
from django.db import models
from django.db.models.signals import post_save
from django.utils.translation import ugettext as _

from exceptions import MobileAppDistributionConfigurationException


log = logging.getLogger(__name__)

class UserInfo(models.Model):
	user = models.OneToOneField(User)
	language = models.CharField(max_length=20, choices=app_dist_settings.LANGUAGES, default=app_dist_settings.ENGLISH)

	def __unicode__(self):
		return _('Extended user info')

	class Meta:
		verbose_name = _('Extended user info')
		verbose_name_plural = _('Extended user info')



class App(models.Model):
	user = models.ForeignKey(User, related_name='apps', verbose_name=_('User'))
	name = models.CharField(max_length=200, verbose_name=_('App name'))
	comment = models.CharField(max_length=200, verbose_name=_('Comment'), blank=True, null=True)
	version = models.CharField(max_length=200, verbose_name=_('Version'))
	build_date = models.DateTimeField(verbose_name=_('Build date'))
	updatedAt = models.DateTimeField(auto_now=True, auto_now_add=True, editable=False)
	createdAt = models.DateTimeField(auto_now_add=True, editable=False)	

	def __unicode__(self):
		return self.name

class IosApp(App):

	file_name = models.CharField(max_length=200, verbose_name=_('File name'))
	operating_system = models.CharField( max_length=50, choices=app_dist_settings.OS_CHOICES, default=app_dist_settings.IOS, verbose_name=_('Operating system'), editable=False)
	app_binary = models.FileField(upload_to=app_dist_settings.MOBILE_APP_DISTRIBUTION_IOS_UPLOAD_TO_DIRECTORY_NAME, verbose_name=_('Ad Hoc ipa file'))
	app_plist = models.FileField(upload_to=app_dist_settings.MOBILE_APP_DISTRIBUTION_IOS_UPLOAD_TO_DIRECTORY_NAME, verbose_name=_('Ad Hoc plist'))

	def get_binary_url(self):
		if not self.app_binary:
			return None
		Site.objects.clear_cache()
		try:
			Site.objects.get_current()
		except Exception:
			raise MobileAppDistributionConfigurationException("The site framework's domain name is used to generate the plist and binary links.  Please configure your current site properly. Also make sure that the SITE_ID in your settings file matches the primary key of your current site.")

		return Site.objects.get_current().domain + self.app_binary.url
			

	def get_plist_url(self):
		if not self.app_plist:
			return None
		Site.objects.clear_cache()
		try:
			Site.objects.get_current()
		except Exception:
			raise MobileAppDistributionConfigurationException("The site framework's domain name is used to generate the plist and binary links.  Please configure your current site properly. Also make sure that the SITE_ID in your settings file matches the primary key of your current site.")
		return Site.objects.get_current().domain + self.app_plist.url

	class Meta:
		verbose_name = _('iOS App')
		verbose_name_plural = _('iOS Apps')
		ordering = ( 'name', 'operating_system', '-version', '-build_date',)


fs = FileSystemStorage(location=app_dist_settings.MOBILE_APP_DISTRIBUTION_ANDROID_FILE_STORAGE_PATH)

class AndroidApp(App):
	operating_system = models.CharField( max_length=50, choices=app_dist_settings.OS_CHOICES, default=app_dist_settings.ANDROID, verbose_name=_('Operating system'), editable=False)
	app_binary = models.FileField(upload_to=app_dist_settings.MOBILE_APP_DISTRIBUTION_ANDROID_UPLOAD_TO_DIRECTORY_NAME, verbose_name=_('APK file'), storage=fs)

	class Meta:
		verbose_name = _('Android App')
		verbose_name_plural = _('Android Apps')
		ordering = ( 'name', 'operating_system', '-version', '-build_date',)


def create_user_info(sender, instance, created, **kwargs):
	try:
		instance.userinfo
	except UserInfo.DoesNotExist:
		UserInfo.objects.create(user=instance)

post_save.connect(create_user_info, sender=User)



from os.path import join
from django.conf import settings

def get(key, default):
  return getattr(settings, key, default)

IOS = 'iOS'
IOS_PLIST = 'iOS Plist'
ANDROID = 'Android'
OS_CHOICES = (
    (IOS, 'iOS'),
    (ANDROID, 'Android'),
)

MOBILE_APP_DISTRIBUTION_CONTENT_TYPES = {
    IOS : 'application/octet-stream ipa',
    IOS_PLIST : 'text/xml plist',
    ANDROID : 'application/vnd.android.package-archive'
}

MOBILE_APP_DISTRIBUTION_IOS_UPLOAD_TO_DIRECTORY_NAME = get('MOBILE_APP_DISTRIBUTION_IOS_UPLOAD_TO_DIRECTORY_NAME', 'ios_apps')
MOBILE_APP_DISTRIBUTION_ANDROID_UPLOAD_TO_DIRECTORY_NAME = get('MOBILE_APP_DISTRIBUTION_ANDROID_UPLOAD_TO_DIRECTORY_NAME', 'android_apps')
MOBILE_APP_DISTRIBUTION_ANDROID_FILE_STORAGE_PATH = join(settings.BASE_PATH, get('MOBILE_APP_DISTRIBUTION_ANDROID_FILE_STORAGE_PATH', 'android'))

ENGLISH = 'en'
GERMAN = 'de'

LANGUAGES = (
	(ENGLISH, 'English'), 
	(GERMAN, 'Deutsch'),
)